@@ -12,7 +12,7 @@ from parsimonious.exceptions import VisitationError
 from parsimonious.utils import StrAndRepr
 
 
-class Node(object):
+class Node(StrAndRepr):
     """A parse tree node
 
     Consider these immutable once constructed. As a side effect of a
@@ -83,9 +83,6 @@ class Node(object):
         """Return a compact, human-readable representation of me."""
         return self.prettily()
 
-    def __str__(self):
-        return unicode(self).encode('utf-8')
-
     def __eq__(self, other):
         """Support by-value deep comparison with other nodes for testing."""
         return (other is not None and
@@ -163,12 +160,11 @@ class NodeVisitor(object):
         except VisitationError:
             # Don't catch and re-wrap already-wrapped exceptions.
             raise
-        except Exception, e:
+        except Exception as e:
             # Catch any exception, and tack on a parse tree so it's easier to
             # see where it went wrong.
             exc_class, exc, tb = sys.exc_info()
-            visitation_exception = VisitationError(exc, exc_class, node)
-            raise visitation_exception.__class__, visitation_exception, tb
+            raise VisitationError, (exc, exc_class, node), tb
 
     def generic_visit(self, node, visited_children):
         """Default visitor method
