@@ -4,7 +4,7 @@ from unittest import TestCase
 from nose import SkipTest
 from nose.tools import eq_, assert_raises, ok_
 
-from parsimonious.exceptions import UndefinedLabel, BadGrammar
+from parsimonious.exceptions import UndefinedLabel, ParseError
 from parsimonious.nodes import Node
 from parsimonious.grammar import rule_grammar, RuleVisitor, Grammar
 
@@ -13,31 +13,39 @@ class BootstrappingGrammarTests(TestCase):
     """Tests for the expressions in the grammar that parses the grammar
     definition syntax"""
 
-    def test_ws(self):
-        text = ' \t\r'
-        eq_(rule_grammar['ws'].parse(text), Node('ws', text, 0, 3))
-
     def test_quantifier(self):
         text = '*'
-        eq_(rule_grammar['quantifier'].parse(text), Node('quantifier', text, 0, 1))
+        eq_(rule_grammar['quantifier'].parse(text),
+            Node('quantifier', text, 0, 1, children=[
+                Node('', text, 0, 1), Node('_', text, 1, 1)]))
         text = '?'
-        eq_(rule_grammar['quantifier'].parse(text), Node('quantifier', text, 0, 1))
+        eq_(rule_grammar['quantifier'].parse(text),
+            Node('quantifier', text, 0, 1, children=[
+                Node('', text, 0, 1), Node('_', text, 1, 1)]))
         text = '+'
-        eq_(rule_grammar['quantifier'].parse(text), Node('quantifier', text, 0, 1))
+        eq_(rule_grammar['quantifier'].parse(text),
+            Node('quantifier', text, 0, 1, children=[
+                Node('', text, 0, 1), Node('_', text, 1, 1)]))
 
-    def test_literal(self):
+    def test_spaceless_literal(self):
         text = '"anything but quotes#$*&^"'
-        eq_(rule_grammar['literal'].parse(text), Node('literal', text, 0, len(text)))
+        eq_(rule_grammar['spaceless_literal'].parse(text),
+            Node('spaceless_literal', text, 0, len(text), children=[
+                Node('', text, 0, len(text))]))
         text = r'''r"\""'''
-        eq_(rule_grammar['literal'].parse(text), Node('literal', text, 0, 5))
+        eq_(rule_grammar['spaceless_literal'].parse(text),
+            Node('spaceless_literal', text, 0, 5, children=[
+                Node('', text, 0, 5)]))
 
     def test_regex(self):
         text = '~"[a-zA-Z_][a-zA-Z_0-9]*"LI'
         eq_(rule_grammar['regex'].parse(text),
             Node('regex', text, 0, len(text), children=[
                  Node('', text, 0, 1),
-                 Node('literal', text, 1, 25),
-                 Node('', text, 25, 27)]))
+                 Node('spaceless_literal', text, 1, 25, children=[
+                     Node('', text, 1, 25)]),
+                 Node('', text, 25, 27),
+                 Node('_', text, 27, 27)]))
 
     def test_successes(self):
         """Make sure the PEG recognition grammar succeeds on various inputs."""
@@ -171,8 +179,8 @@ class GrammarTests(TestCase):
                                          Node('bold_close', s, 6, 8)]))
 
     def test_bad_grammar(self):
-        """Constructing a Grammar with bad rules should raise BadGrammar."""
-        assert_raises(BadGrammar, Grammar, 'just a bunch of junk')
+        """Constructing a Grammar with bad rules should raise ParseError."""
+        assert_raises(ParseError, Grammar, 'just a bunch of junk')
 
     def test_comments(self):
         """Test tolerance of comments and blank lines in and around rules."""
@@ -189,22 +197,33 @@ class GrammarTests(TestCase):
                                           # \n or \r to end.
         eq_(list(sorted(str(grammar).splitlines())),
             ['''bold_text = stars text stars''',
-             # Unicode flag is on by default in Python 3. I wonder
-             # if we should turn it on all the time in
-             # Parsimonious.
+             # TODO: Unicode flag is on by default in Python 3. I wonder if we
+             # should turn it on all the time in Parsimonious.
              '''stars = "**"''',
              '''text = ~"[A-Z 0-9]*"i%s''' % ('u' if version_info >= (3,)
                                               else '')])
 
+    def test_multi_line(self):
+        """Make sure we tolerate all sorts of crazy line breaks and comments in
+        the middle of rules."""
+        grammar = Grammar("""
+            bold_text  = bold_open  # commenty comment
+                         text  # more comment
+                         bold_close
+            text       = ~"[A-Z 0-9]*"i
+            bold_open  = "((" bold_close =  "))"
+            """)
+        ok_(grammar.parse('((booyah))') is not None)
+
     def test_not(self):
         """Make sure "not" predicates get parsed and work properly."""
         grammar = Grammar(r'''not_arp = !"arp" ~"[a-z]+"''')
-        eq_(grammar.parse('arp'), None)
+        assert_raises(ParseError, grammar.parse, 'arp')
         ok_(grammar.parse('argle') is not None)
 
     def test_lookahead(self):
         grammar = Grammar(r'''starts_with_a = &"a" ~"[a-z]+"''')
-        eq_(grammar.parse('burp'), None)
+        assert_raises(ParseError, grammar.parse, 'burp')
 
         s = 'arp'
         eq_(grammar.parse('arp'), Node('starts_with_a', s, 0, 3, children=[
@@ -214,7 +233,7 @@ class GrammarTests(TestCase):
     def test_parens(self):
         grammar = Grammar(r'''sequence = "chitty" (" " "bang")+''')
         # Make sure it's not as if the parens aren't there:
-        eq_(grammar.parse('chitty bangbang'), None)
+        assert_raises(ParseError, grammar.parse, 'chitty bangbang')
 
         s = 'chitty bang bang'
         eq_(str(grammar.parse(s)),
@@ -236,7 +255,7 @@ class GrammarTests(TestCase):
             term = number
             number = ~r"[0-9]+"
             """)
-        grammar.parse('(3 4)')
+        grammar.parse('(34)')
 
     def test_infinite_loop(self):
         """Smoke-test a grammar that was causing infinite loops while building.
@@ -269,3 +288,11 @@ class GrammarTests(TestCase):
             foo = bar
             bar = foo
             """)
+
+    def test_parens_with_leading_whitespace(self):
+        """Make sure a parenthesized expression is allowed to have leading
+        whitespace when nested directly inside another."""
+        Grammar("""foo = ( ("c") )""").parse('c')
+
+    def test_single_quoted_literals(self):
+        Grammar("""foo = 'a' '"'""").parse('a"')
